@@ -43,6 +43,8 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <stdio.h>
+#include <locale.h>
+#include <stdlib.h>
 
 #include <stdexcept>
 
@@ -177,6 +179,8 @@ CTemplate::CTemplate(const char *src, SourceType type,
     core_ = reinterpret_cast<CTPP::VMExecutable *>(malloc(core_size_));
     memcpy(core_, prg_core, core_size_);
     vm_mem_core_ = new CTPP::VMMemoryCore(core_);
+
+    if (type == kFileSourceType) name_ = file_loader.GetTemplateName();
   }
 }
 
@@ -185,10 +189,26 @@ CTemplate::~CTemplate() throw() {
   free(core_);
 }
 
-bool CTemplate::Render(CTPP::CDT *params, std::string *result) {
+bool CTemplate::Render(CTPP::CDT *params, const char *locale,
+                       std::string *result) {
   uint32_t i_pointer = 0;
   std::string res;
 
+  if (locale) {
+    if (!setlocale(LC_ALL, locale)) {
+      last_error_type_ = kSystemErrorType;
+      last_error_message_ = string("Failed to set locale") + ": " + name_;
+      return false;
+    }
+
+    if (setenv("LC_ALL", locale, 1) < 0) {
+      last_error_type_ = kSystemErrorType;
+      last_error_message_ = string("Failed to set locale (") + strerror(errno)
+          + ")" + ": " + name_;
+      return false;
+    }
+  }
+
   try {
     CTPP::StringOutputCollector out_collector(*result);
     PyLogger logger;
@@ -199,15 +219,15 @@ bool CTemplate::Render(CTPP::CDT *params, std::string *result) {
     vm.Run(vm_mem_core_, &out_collector, i_pointer, *params, &logger);
   } catch (CTPP::CTPPException &e) {
     last_error_type_ = kIOErrorType;
-    last_error_message_ = e.what();
+    last_error_message_ = string(e.what()) + ": " + name_;
     return false;
   } catch (std::exception &e) {
     last_error_type_ = kIOErrorType;
-    last_error_message_ = e.what();
+    last_error_message_ = string(e.what()) + ": " + name_;
     return false;
   } catch (...) {
     last_error_type_ = kSystemErrorType;
-    last_error_message_ = "Unknown error occured";
+    last_error_message_ = string("Unknown error occured") + ": " + name_;
     return false;
   }
 
@@ -215,10 +235,11 @@ bool CTemplate::Render(CTPP::CDT *params, std::string *result) {
 }
 
 bool CTemplate::SaveBytecode(const char *filename) {
-  FILE *file = fopen(filename, "w");
+  FILE *file = fopen(filename, "a");
   if (!file) {
     last_error_type_ = kIOErrorType;
-    last_error_message_ = string("Can't save bytecode: ") + strerror(errno);
+    last_error_message_ =
+        string("Can't save bytecode: ") + strerror(errno) + ": " + filename;
     return false;
   }
 
